#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <ctime>

#define LINE_SIZE 5000
int * used = NULL;

void generate_zitzler (int size, int knapsacks, int amount, int lower_bound, int upper_bound, float ratio)
{
   int i, j, z;
   FILE * Out;
   char basename[1000], fname[1000];

   int * profits = (int *) calloc(sizeof(int), size);
   int * weights = (int *) calloc(sizeof(int), size);
   int capacity = 0;

   sprintf(basename, "knapsack.%d.%d", size, knapsacks);

   for (i = 0; i < amount; ++i)
   {

	if (!used) sprintf(fname, "%s.%d.out", basename, i + 1);
	else sprintf(fname, "%s.%d.out", basename, used[size - 100]);

	fprintf(stderr, "Generating file %s...\n", fname);

	Out = fopen(fname, "w");
	fprintf(Out, "knapsack problem specification (%d knapsacks, %d items)\n", knapsacks, size);

	for (j = 0; j < knapsacks; ++j)
	{
	   capacity = 0;
	   fprintf(Out, "=\n");

	   for (z = 0; z < size; ++z)
	   {
	      profits[z] = rand() % (upper_bound - lower_bound + 1) + lower_bound;
	      weights[z] = rand() % (upper_bound - lower_bound + 1) + lower_bound;
	      capacity += weights[z];
	   }
	   capacity /= 2;

	   fprintf(Out, "knapsack %d:\n", j + 1);
	   fprintf(Out, " capacity: +%d\n", capacity);

	   for (z = 0; z < size; ++z)
	   {
	      fprintf(Out, " item %d:\n", z+1);
	      fprintf(Out, " weight: +%d\n", weights[z]);
	      fprintf(Out, " profit: +%d\n", profits[z]);
	   }
	}
	fclose(Out);
   }
}

void usage (char * program) 
{
     fprintf(stderr, "Syntax error! Usage: %s [-f <param_file>] <lower_bound> <upper_bound> <nobj> <ratio> <ninstances> <nitems> [-r <lower_bound> <upper_bound>]\n\n"
			"\t-f <param_file> 		\t   reads parameters from a file. Ignores other parameters.\n"
			"\t-r <lower_bound> <upper_bound>   \t randomly choose number of itens within the given bounds.\n", program); 
}

int main (int argc, char * argv[]) 
{
  
  char buffer[LINE_SIZE];
  int lower_bound, upper_bound, size, amount, objectives, dimensions, lb_items, ub_items;
  float ratio;
  bool random = false;
 
  if (argc < 3) { usage(argv[0]); abort(); }

  int seed = (int) time(NULL);
  srand(seed);

  FILE * Log = fopen("knapsack.log", "w");
  fprintf(Log, "seed: %d\n", seed);
  fclose(Log);	

  FILE * Params;
  if (!strcmp(argv[1],"-f")) {

    Params = fopen(argv[2], "r");  

    assert(fgets(buffer, LINE_SIZE, Params) > 0);
    if (strcmp(buffer, "zitzler\n")) { fprintf(stderr, "Error! Method chosen: %s\nMethods currently available: zitzler.\n", buffer); abort(); }

    assert(fgets(buffer, LINE_SIZE, Params) > 0);
    sscanf(buffer, "%d %d", &lower_bound, &upper_bound);

    assert(fgets(buffer, LINE_SIZE, Params) > 0);
    sscanf(buffer, "%d", &objectives);
  
    assert(fgets(buffer, LINE_SIZE, Params) > 0);
    sscanf(buffer, "%f", &ratio);
  
    while (fgets(buffer, LINE_SIZE, Params) > 0)
    {
	sscanf(buffer, "%d %d", &size, &amount);
	generate_zitzler(size, objectives, amount, lower_bound, upper_bound, ratio);
    }

  }

  else
  {
    int i = 1;
    if (!strcmp(argv[1],"-r")) 
    {
	random = true;
    	assert(argc >= 8);
    	lb_items = atoi(argv[++i]);
    	ub_items = atoi(argv[++i]);
	i++;
    }
    else assert(argc >= 5);

    lower_bound = atoi(argv[i++]);
    upper_bound = atoi(argv[i++]);
    objectives = atoi(argv[i++]);
    ratio = atof(argv[i++]);
    amount = atoi(argv[i++]);

    if (!random && !strcmp(argv[i],"-r"))
    {
	random = true;
    	assert(argc >= 8);
    	lb_items = atoi(argv[++i]);
    	ub_items = atoi(argv[++i]);
    }

    fprintf(stdout, "%d %d %d %f %d %s %d %d\n", lower_bound, upper_bound, objectives, ratio, amount, random ? "true" : "false", lb_items, ub_items); 
    if (!random) 
    {
	size = atoi(argv[i++]);
	generate_zitzler(size, objectives, 1, lower_bound, upper_bound, ratio);
    }
    else 
    {
	used = (int *) calloc(sizeof(int), ub_items - lb_items + 1);
	for (i = 0; i < amount; ++i)
	{
	   size = rand() % (ub_items-lb_items+1) + lb_items;
	   used[size-lb_items]++;
	   fprintf(stderr,"size = %d, used[%d] = %d\n", size, size-lb_items, used[size-lb_items]);
	   generate_zitzler(size, objectives, 1, lower_bound, upper_bound, ratio);
	}
    }
    
  }

  return 1;
}
